///////////////////////////////////////////////////////////
//                     GGBuildTools                      //
//         Gipat Group       //
//             Copyright (C) 2007 Gipat Group            //
//                             //
//    Gipat Group's opened EI-editor-utility license     //
//                       1.0                       //
//                                                       //
//                  www.gipatgroup.org                   //
///////////////////////////////////////////////////////////

//      , ....  :
// 1) Sagrer (sagrer@yandex.ru)

////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////
//            *.iss        //
//                  //
//           InnoSetup compiler           //
////////////////////////////////////////////////////

unit IssFormat;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LCLAnIniFile, ExtraFunctionsLcl, FlistFormat,
  PascalCodeModifier, ExtraFileUtilsLcl;
  
const
  // -...               .
  IssF_CurrMainComponentName = 'CurrVersion_Main';    //   ""  .
  IssF_AlwaysInstallComponentName = 'AlwaysInstall';  //    .
  IssF_FilesAddFlags = true;        //     .
  IssF_DirsAddFlags = false;        //     .
  IssF_FilesAddingFlags = 'overwritereadonly replacesameversion sortfilesbyextension';    // ,    .
  IssF_DirsAddingFlags = '';                                         // ,    .
  //   [Code]  .
  IssF_PreCode = 'Program InnoScript;'+#13+#10;     //   .
  IssF_PostCode = #13+#10+'begin end.';             //   .
  
type
  TIssFormat = class (TAnIniFile)              //     (*.iss)  Inno Setup Compiler.       ...
  private
    PasModifier : TPascalCodeModifier;
    PasModifIsActual : Boolean;
    // .
    function ElementListHas(const ElementList, CheckableComponent : AnsiString) : Boolean;   // -      .
    function UpdatePasModifier() : Boolean;           // PasModifier    [Code].
    function SaveFromPasModifier() : Boolean;         //   PasModifier   [Code].
  public
    //.
    ErrorMessage : AnsiString;
    
    //-...
    Constructor Create; override;
    Destructor Destroy; override;
    
    // .
    procedure MakNewFile(); override;   //  .
    Procedure RemoveSection(const SectName : string); override;   //  .
    Procedure WriteRawSection(const SectName, SectContent : string); override; //     .
    procedure ClearFiles();             //    -  ..   .
    procedure ClearIcons();             //   .
    procedure ClearRegistry();          //   .
    function UpdateInfoToVerUpdate(const OldVerIssFile, OldVerCompName : AnsiString) : Boolean;   //            .
    function AddFilesFromFlist(var FlistFile : TFlistFormat ; const ComponentName : AnsiString) : Boolean;        //    Flist-   .
    function AddFilesFromFlist(const FlistFileName, ComponentName : AnsiString) : Boolean;        //    Flist-   .
    function AddFilesFromDir(const DirName, ComponentName : AnsiString) : Boolean;   //    Flist-a   .
    function AddNewFilesFromFlists(var OldFlistFile, NewFlistFile : TFlistFormat; const ComponentName : AnsiString) : Boolean;   //        FList-   .
    function AddNewFilesFromFlists(const OldFlistFileName, NewFlistFileName, ComponentName : AnsiString) : Boolean;     //        FList-   .
    function AddNewFilesFromDirs(const OldDirName, NewDirName, ComponentName : AnsiString) : Boolean;    //           .
    function ReadCodeConstValue(const ConstName : AnsiString) : AnsiString;  //   .
    function WriteCodeConstValue(const ConstName, ConstValue : AnsiString) : Boolean;    //   .
    function CodeFunctionExists(const FunctionName : AnsiString) : Boolean;   //     ?
    function WriteCodeFunction(const FunctionName, FunctionContent : AnsiString) : Boolean;   //   Code,    - .
    function InstallTypeExists(const InstallTypeName : AnsiString) : Boolean;   //   ?
    function RemoveInstalType(const InstallTypeName : AnsiString) : Boolean;   //  . false        ,   true.
    procedure WriteInstallType(const InstallTypeName, FullInstTypeString : AnsiString);    //  ,   -      . FullInstTypeString -     [Types].
    function ComponentExists(const ComponentName : AnsiString) : Boolean;    //   ?
    function RemoveComponent(const ComponentName : AnsiString) : Boolean;    // . false       ,      true.
    procedure WriteComponent(const ComponentName, FullComponentString : AnsiString);   // ,   -      . FullComponentString -     [Components].
    //   \ -   .
    function GetSourceDir() : AnsiString;          // SourceDir.
    function GetOutputDir() : AnsiString;          // OutputDir.
    procedure SetSourceDir(const SourceDir : AnsiString);     // SourceDir.
    procedure SetOutputDir(const OutputDir : AnsiString);     // OutputDir.
  end;

implementation

/////////////////////////////////////////////
//              TIssFormat                 //
/////////////////////////////////////////////

//-----------------------------------------//
//        -...      //
//-----------------------------------------//

Constructor TIssFormat.Create;
begin
  //.   .
  inherited;
  
  //  .
  Self.PasModifier := TPascalCodeModifier.Create;
  
  //  .
  Self.MakNewFile();
end;

Destructor TIssFormat.Destroy;
begin
  // .
  Self.MakNewFile();
  inherited;
end;

//------------------------------------------//
//            ...             //
//------------------------------------------//

function TIssFormat.ElementListHas(const ElementList, CheckableComponent : AnsiString) : Boolean;
// -      .
var
  TempList, TempValue, TempCheck : AnsiString;
begin
  //.
  Result := false;
  TempCheck := LowerCase(CheckableComponent);
  TempList := Trim(ElementList);
  
  //...
  repeat
    if TempList <> '' then begin
      TempValue := LowerCase(Parse(TempList,' '));
      TempList := Trim(TempList);
      if TempValue = TempCheck then begin
        //.
        Result := true;
        TempList := '';
      end;
    end;
  until TempList = '';
  
end;

function TIssFormat.UpdatePasModifier() : Boolean;
// PasModifier    [Code].
begin
  //.
  Result := true;
  
  // .
  if Self.PasModifIsActual = false then begin
    //   .
    if Self.SectionExists('Code') = true then begin
      //  . .
      Self.PasModifier.MakNewFile;
      Self.PasModifier.FileString := IssF_PreCode+Self.ReadRawSection('Code')+IssF_PostCode;
      try
        if Self.PasModifier.ReParseFile() = false then begin
          //        ReParseFile.
          Self.ErrorMessage := 'TPascalCodeModifier, while parsing error: '+Self.PasModifier.ErrorMessage;
          Result := false;
          Exit;
        end;
      except
        //   
        on E: Exception do begin
          Self.ErrorMessage := 'TPascalCodeModifier, while parsing error: '+E.Message;
          Result := false;
          Exit;
        end;
      end;
      
      //.
      Self.PasModifIsActual := true;
    end;
  end;
end;

function TIssFormat.SaveFromPasModifier() : Boolean;
//   PasModifier   [Code].
begin
  //.
  Result := true;
  
  if Self.PasModifIsActual = true then begin
    //,   . .
    Self.WriteRawSection('Code', Copy(Self.PasModifier.FileString, Length(IssF_PreCode)+1, Length(Self.PasModifier.FileString)-(Length(IssF_PreCode)+Length(IssF_PostCode))) );
    Self.PasModifIsActual := true;
  end;
end;

//------------------------------------------//
//             ...            //
//------------------------------------------//

procedure TIssFormat.MakNewFile();
//  .
begin
  inherited;
  Self.PasModifier.MakNewFile;
  Self.PasModifIsActual := false;
  ErrorMessage := '';
end;

Procedure TIssFormat.RemoveSection(const SectName : string);
//  .
begin
  //  -     [Code].
  //  ,  ...
  inherited;
  
  //   [Code]   .
  if (LowerCase(SectName) = 'code') and (Self.PasModifIsActual = true) then begin
    Self.PasModifIsActual := false;
  end;
end;

Procedure TIssFormat.WriteRawSection(const SectName, SectContent : string);
//     .
begin
  //  -     [Code].
  //  ,  ...
  inherited;
  
  //   [Code]   .
  if (LowerCase(SectName) = 'code') and (Self.PasModifIsActual = true) then begin
    Self.PasModifIsActual := false;
  end;
end;

procedure TIssFormat.ClearFiles();
//    -  ..   .
begin
  // -       [Dirs], [Files], [InstallDelete].  .
  Self.WriteRawSection('Dirs','');
  Self.WriteRawSection('Files','');
  Self.WriteRawSection('InstallDelete','');
end;

procedure TIssFormat.ClearIcons();
//   .
begin
  //      [Icons].
  Self.WriteRawSection('Icons','');
end;

procedure TIssFormat.ClearRegistry();
//   .
begin
  //      [Registry].
  Self.WriteRawSection('Registry','');
end;

function TIssFormat.UpdateInfoToVerUpdate(const OldVerIssFile, OldVerCompName : AnsiString) : Boolean;
//            .
var
  OldIconList, NewIconList, DeleteList, OldRegistryList, NewRegistryList : AIniValueList;
  OldIssFile : TIssFormat;
  I, oI : Integer;
  NewCheckableElem, ElemIsAddable : Boolean;
  
begin
  //...
  Result := false;
  OldIssFile := TIssFormat.Create;
  SetLength(OldIconList,0,0);
  SetLength(NewIconList,0,0);
  SetLength(DeleteList,0,0);
  SetLength(OldRegistryList,0,0);
  SetLength(NewRegistryList,0,0);
  
  //  "" ...
  if OldIssFile.Load(OldVerIssFile) = true then begin
    //      ...
    Self.ReadIniValueList('Icons',NewIconList);
    //.     "" ...
    OldIssFile.ReadIniValueList('Icons',OldIconList);
    //.        ...
    Self.ReadIniValueList('InstallDelete',DeleteList);
    //      ...
    Self.ReadIniValueList('Registry',NewRegistryList);
    //     "" .
    OldIssFile.ReadIniValueList('Registry',OldRegistryList);
    
    // :
    if (Length(OldIconList) > 0) and (Length(NewIconList) > 0) then begin
      //      - ...

      //  -          ...
      //     (     ) -     "CurrVersion_Main" -           .
      for I := 0 to Length(NewIconList)-1 do begin
        // -       .
        ElemIsAddable := false;
        // -     "" .
        NewCheckableElem := ElementListHas(GetIniValueListValue(NewIconList,I,'Components'),IssF_CurrMainComponentName);

        //      -   -        ,      .
        if NewCheckableElem = true then begin
          // ...
          ElemIsAddable := true;   //         ,     .
          //   .
          for oI := 0 to Length(OldIconList)-1 do begin
            //  .           ,   .       .
            //       .
            if Self.CompareIniValueElements(NewIconList,OldIconList, I, oI, true, 'Filename') = true then begin
              if Self.CompareIniValueElements(NewIconList,OldIconList, I, oI, false, 'all, Filename') = true then begin
                // -      .
                ElemIsAddable := false;
              end;
            end;
          end;
        end;
        
        //   ,     - .
        if ElemIsAddable = true then begin
          //  ...
          SetLength(NewIconList,Length(NewIconList)+1);
          // ...
          Self.CopyIniValueListElement(NewIconList,NewIconList,I,Length(NewIconList)-1);
          //  ...
          Self.ChangeIniValueListValue(NewIconList,Length(NewIconList)-1,'Components',OldVerCompName);
        end;
      end;
      
      //, .          ,     ,    .
      for I := 0 to Length(OldIconList)-1 do begin
        // -    (   )    .
        ElemIsAddable := false;
        // -     "" ,   AlwaysInstall,       .
        NewCheckableElem := (ElementListHas(GetIniValueListValue(OldIconList,I,'Components'),IssF_CurrMainComponentName)) or (ElementListHas(GetIniValueListValue(OldIconList,I,'Components'),IssF_AlwaysInstallComponentName));

        //      -   -        ,   -    .
        if NewCheckableElem = true then begin
          // ...
          ElemIsAddable := true;   //         ...
          //  "" .
          for oI := 0 to Length(NewIconList)-1 do begin
            //  .           ,   .       .
            //       .
            if Self.CompareIniValueElements(OldIconList,NewIconList, I, oI, true, 'Filename') = true then begin
              if Self.CompareIniValueElements(OldIconList,NewIconList, I, oI, false, 'all, Filename') = true then begin
                // -    .
                ElemIsAddable := false;
              end;
            end;
          end;
        end;

        //   ,        - .
        if ElemIsAddable = true then begin
          //  ...
          SetLength(DeleteList,Length(DeleteList)+1);
          //    3   :
          SetLength(DeleteList[Length(DeleteList)-1],3);
          DeleteList[Length(DeleteList)-1,0].Name := 'Name';
          DeleteList[Length(DeleteList)-1,0].Value := GetIniValueListValue(OldIconList,I,'Name');
          DeleteList[Length(DeleteList)-1,1].Name := 'Type';
          DeleteList[Length(DeleteList)-1,1].Value := 'files';
          DeleteList[Length(DeleteList)-1,2].Name := 'Components';
          DeleteList[Length(DeleteList)-1,2].Value := OldVerCompName;
        end;

      end;
      
      //,      .     (NewIconList  DeleteList)  -     .
      Self.WriteIniValueList('Icons',NewIconList);
      Self.WriteIniValueList('InstallDelete',DeleteList);
    end;
    
    // .  ,    ...
    if (Length(OldRegistryList) > 0) and (Length(NewRegistryList) > 0) then begin
      //  -       ,    ...
      for I := 0 to Length(NewRegistryList)-1 do begin
        // -       .
        ElemIsAddable := false;
        // -     "" .
        NewCheckableElem := ElementListHas(GetIniValueListValue(NewRegistryList,I,'Components'),IssF_CurrMainComponentName);

        //      -   -        ,      .
        if NewCheckableElem = true then begin
          // ...
          ElemIsAddable := true;   //         ,     .
          //   .
          for oI := 0 to Length(OldRegistryList)-1 do begin
            //  .    %)        ,   .       .
            if Self.CompareIniValueElements(NewRegistryList,OldRegistryList, I, oI, true, '') = true then begin
              // -      .
              ElemIsAddable := false;
            end;
          end;
        end;

        //   ,     - .
        if ElemIsAddable = true then begin
          //  ...
          SetLength(NewRegistryList,Length(NewRegistryList)+1);
          // ...
          Self.CopyIniValueListElement(NewRegistryList,NewRegistryList,I,Length(NewRegistryList)-1);
          //  ...
          Self.ChangeIniValueListValue(NewRegistryList,Length(NewRegistryList)-1,'Components',OldVerCompName);
        end;
      end;
      
      //, .          ,     ,    ,    ,   ( none) .
      for I := 0 to Length(OldRegistryList)-1 do begin
        // -         .
        ElemIsAddable := false;
        // -     "" ,   AlwaysInstall,       .
        NewCheckableElem := (ElementListHas(GetIniValueListValue(OldRegistryList,I,'Components'),IssF_CurrMainComponentName)) or (ElementListHas(GetIniValueListValue(OldRegistryList,I,'Components'),IssF_AlwaysInstallComponentName));

        //      -   -        ,   -     (,    ,      ,        ).
        if NewCheckableElem = true then begin
          // ...
          ElemIsAddable := true;   //         ...
          //  "" .
          for oI := 0 to Length(NewRegistryList)-1 do begin
            //  .         ,   .       .
            if Self.CompareIniValueElements(OldRegistryList,NewRegistryList, I, oI, true, '') = true then begin
              // -    .
              ElemIsAddable := false;
            end;
          end;
        end;

        //   ,        - .
        if ElemIsAddable = true then begin
          //         -      -  % ))).
          //  ?
          if Self.IniValueListValueExists(OldRegistryList,I,'ValueType') = false then begin
            //,  .       ?
            if ElementListHas(GetIniValueListValue(OldRegistryList,I,'Flags'),'deletekey') = false then begin
              //, .   .     -:
              //  ...
              SetLength(NewRegistryList,Length(NewRegistryList)+1);
              //    4   :
              SetLength(NewRegistryList[Length(NewRegistryList)-1],4);
              NewRegistryList[Length(NewRegistryList)-1,0].Name := 'Root';
              NewRegistryList[Length(NewRegistryList)-1,0].Value := GetIniValueListValue(OldRegistryList,I,'Root');
              NewRegistryList[Length(NewRegistryList)-1,1].Name := 'Subkey';
              NewRegistryList[Length(NewRegistryList)-1,1].Value := GetIniValueListValue(OldRegistryList,I,'Subkey');
              NewRegistryList[Length(NewRegistryList)-1,2].Name := 'Flags';
              NewRegistryList[Length(NewRegistryList)-1,2].Value := 'deletekey';
              NewRegistryList[Length(NewRegistryList)-1,3].Name := 'Components';
              NewRegistryList[Length(NewRegistryList)-1,3].Value := OldVerCompName;
            end
            else begin
              //, ,      ,  ,     .
            end;
          end
          else begin
            //,  .   none   ?
            if LowerCase(GetIniValueListValue(OldRegistryList,I,'ValueType')) <> 'none' then begin
              //,   none - ,        ?
              if ElementListHas(GetIniValueListValue(OldRegistryList,I,'Flags'),'deletevalue') = false then begin
                //,     -   :
                //  ...
                SetLength(NewRegistryList,Length(NewRegistryList)+1);
                //    6   :
                SetLength(NewRegistryList[Length(NewRegistryList)-1],6);
                NewRegistryList[Length(NewRegistryList)-1,0].Name := 'Root';
                NewRegistryList[Length(NewRegistryList)-1,0].Value := GetIniValueListValue(OldRegistryList,I,'Root');
                NewRegistryList[Length(NewRegistryList)-1,1].Name := 'Subkey';
                NewRegistryList[Length(NewRegistryList)-1,1].Value := GetIniValueListValue(OldRegistryList,I,'Subkey');
                NewRegistryList[Length(NewRegistryList)-1,2].Name := 'ValueType';
                NewRegistryList[Length(NewRegistryList)-1,2].Value := 'none';
                NewRegistryList[Length(NewRegistryList)-1,3].Name := 'ValueName';
                NewRegistryList[Length(NewRegistryList)-1,3].Value := GetIniValueListValue(OldRegistryList,I,'ValueName');
                NewRegistryList[Length(NewRegistryList)-1,4].Name := 'Flags';
                NewRegistryList[Length(NewRegistryList)-1,4].Value := 'deletevalue';
                NewRegistryList[Length(NewRegistryList)-1,5].Name := 'Components';
                NewRegistryList[Length(NewRegistryList)-1,5].Value := OldVerCompName;
              end
              else begin
                //   -    .
              end;
            end
            else begin
              //   none      -         ,  
              //      (      ),      
              //     -      ,      .
            end;
          end;
        end;
      end;
      
      //,      .      (NewRegistryList)  -     .
      Self.WriteIniValueList('Registry',NewRegistryList);
    end;
  end;
  
  //.
  Result := true;
  
  // ...
  OldIssFile.Free;
  SetLength(OldIconList,0,0);
  SetLength(NewIconList,0,0);
  SetLength(DeleteList,0,0);
  SetLength(OldRegistryList,0,0);
  SetLength(NewRegistryList,0,0);
end;

function TIssFormat.AddFilesFromFlist(var FlistFile : TFlistFormat ; const ComponentName : AnsiString) : Boolean;
//    Flist-   .
var
  DirsList, FilesList : AIniValueList;
  RealDirsListSize, RealFilesListSize, I, ElemsNum : Integer;
  TempStr : AnsiString;
  
begin
  //...
  Result := false;
  SetLength(DirsList,0,0);
  SetLength(FilesList,0,0);

  //       ...
  Self.ReadIniValueList('Dirs',DirsList);
  Self.ReadIniValueList('Files',FilesList);
  //    ,      flist.
  RealDirsListSize := Length(DirsList);
  RealFilesListSize := Length(FilesList);
  SetLength(DirsList,RealDirsListSize+FlistFile.FilesNum);
  SetLength(FilesList,RealFilesListSize+FlistFile.FilesNum);
  //,     flist-.
  for I := 0 to FlistFile.FilesNum-1 do begin
    //    -   ?
    if FlistFile.FileList[I].FileType = Flist_Filetype_File then begin
      //. .      .
      ElemsNum := 3;  // .
      if IssF_FilesAddFlags = true then begin
        // .
        ElemsNum := ElemsNum+1;
      end;
      //  ..
      SetLength(FilesList[RealFilesListSize],ElemsNum);
      //    ...
      TempStr := GetLocalPath(FlistFile.GetGlobalRoot(),FlistFile.GetRealElementName(I));
      FilesList[RealFilesListSize,0].Name := 'Source';
      FilesList[RealFilesListSize,0].Value := RightStr(TempStr,Length(TempStr)-2);
      FilesList[RealFilesListSize,1].Name := 'DestDir';
      FilesList[RealFilesListSize,1].Value := AddLastSlash('{app}'+ExtractFilePath(RightStr(TempStr,Length(TempStr)-1)));
      FilesList[RealFilesListSize,2].Name := 'Components';
      FilesList[RealFilesListSize,2].Value := ComponentName;
      if IssF_FilesAddFlags = true then begin
        FilesList[RealFilesListSize,3].Name := 'Flags';
        FilesList[RealFilesListSize,3].Value := IssF_FilesAddingFlags;
      end;
      //     .
      RealFilesListSize := RealFilesListSize+1;
    end
    else if FlistFile.FileList[I].FileType = Flist_Filetype_Directory then begin
      //.      ?
      TempStr := GetLocalPath(FlistFile.GetGlobalRoot(),FlistFile.GetRealElementName(I));
      if Length(TempStr) > 2 then begin
        // ,  . .      .
        ElemsNum := 2;  // .
        if IssF_DirsAddFlags = true then begin
          // .
          ElemsNum := ElemsNum+1;
        end;
        //  ...
        SetLength(DirsList[RealDirsListSize],ElemsNum);
        //    ...
        DirsList[RealDirsListSize,0].Name := 'Name';
        DirsList[RealDirsListSize,0].Value := DelLastSlash('{app}'+RightStr(TempStr,Length(TempStr)-1));
        DirsList[RealDirsListSize,1].Name := 'Components';
        DirsList[RealDirsListSize,1].Value := ComponentName;
        if IssF_DirsAddFlags = true then begin
          DirsList[RealDirsListSize,2].Name := 'Flags';
          DirsList[RealDirsListSize,2].Value := IssF_DirsAddingFlags;
        end;
        //     .
        RealDirsListSize := RealDirsListSize+1;
      end;
    end;
  end;
  
  // .       ...
  SetLength(DirsList,RealDirsListSize);
  SetLength(FilesList,RealFilesListSize);
  
  //     .
  Self.WriteIniValueList('Dirs',DirsList);
  Self.WriteIniValueList('Files',FilesList);
  
  // ...
  Result := true;

  // ...
  SetLength(DirsList,0,0);
  SetLength(FilesList,0,0);
end;

function TIssFormat.AddFilesFromFlist(const FlistFileName, ComponentName : AnsiString) : Boolean;
//    Flist-   .
var
  FlistFile : TFlistFormat;
begin
  //...
  Result := false;
  FlistFile := TFlistFormat.Create;
  
  // flist-  -  .
  if FileExists(FlistFileName) = true then begin
    // .
    FlistFile.Load(FlistFileName);
    //  Flist-.
    FlistFile.GlobalRoot := Self.GetSourceDir();
    //    .
    Result := Self.AddFilesFromFlist(FlistFile,ComponentName);
  end;
  
  // ...
  FlistFile.Free;
end;

function TIssFormat.AddFilesFromDir(const DirName, ComponentName : AnsiString) : Boolean;
//    Flist-a   .
var
  FlistFile : TFlistFormat;
begin
  //...
  Result := false;
  FlistFile := TFlistFormat.Create;

  //...      -              .
  FlistFile.PathsAreAbsolute := true;
  FlistFile.AddSize := false;
  FlistFile.AddCRC32 := false;
  FlistFile.GlobalRoot := Self.GetSourceDir();
  
  //      flist-.
  if FlistFile.AddRealDirectory(DirName,true) = true then begin
    //    -     .
    Result := Self.AddFilesFromFlist(FlistFile,ComponentName);
  end;

  // ...
  FlistFile.Free;
end;

function TIssFormat.AddNewFilesFromFlists(var OldFlistFile, NewFlistFile : TFlistFormat; const ComponentName : AnsiString) : Boolean;
//        FList-   .
var
  ComparedList : AComparedList;
  DirsList, FilesList, DeleteList, DirDeleteList : AIniValueList;
  RealDirsListSize, RealFilesListSize, RealDeleteListSize, RealDirDeleteListSize, I, ElemsNum : Integer;
  //TestStr : AnsiString;
  
begin
  //.
  Result := false;
  SetLength(ComparedList,0);
  SetLength(DirsList,0,0);
  SetLength(FilesList,0,0);
  SetLength(DeleteList,0,0);
  SetLength(DirDeleteList,0,0);
  
  //    ...
  Self.ReadIniValueList('Dirs',DirsList);
  Self.ReadIniValueList('Files',FilesList);
  Self.ReadIniValueList('InstallDelete',DeleteList);
  
  //C .
  OldFlistFile.CompareWithFlist(NewFlistFile,'.\',ComparedList);
  //          .
  OldFlistFile.ConvertToSimpleComparedList(ComparedList);
  
  //    ,      ComparedList.
  RealDirsListSize := Length(DirsList);
  RealFilesListSize := Length(FilesList);
  RealDeleteListSize := Length(DeleteList);
  RealDirDeleteListSize := 0;
  SetLength(DirsList,RealDirsListSize+Length(ComparedList));
  SetLength(FilesList,RealFilesListSize+Length(ComparedList));
  SetLength(DeleteList,RealDeleteListSize+Length(ComparedList));
  SetLength(DirDeleteList,Length(ComparedList));
  
  //   ,     \    .
  for I := 0 to Length(ComparedList)-1 do begin
    //TestStr := ComparedList[I].FilePath;
    if ComparedList[I].CheckResult <> Flist_CheckFile_Identical then begin
      // .  -   .   -    .
      if ComparedList[I].FileType = Flist_Filetype_File then begin
        //.      .
        if (ComparedList[I].CheckResult = Flist_CheckFile_NonIdentical) or (ComparedList[I].CheckResult = Flist_CheckFile_ElementNotExists) then begin
          //  ,        -    .
          //     .
          ElemsNum := 3;  // .
          if IssF_FilesAddFlags = true then begin
            // .
            ElemsNum := ElemsNum+1;
          end;
          //  ..
          SetLength(FilesList[RealFilesListSize],ElemsNum);
          //    ...
          FilesList[RealFilesListSize,0].Name := 'Source';
          FilesList[RealFilesListSize,0].Value := RightStr(ComparedList[I].FilePath,Length(ComparedList[I].FilePath)-2);
          FilesList[RealFilesListSize,1].Name := 'DestDir';
          FilesList[RealFilesListSize,1].Value := AddLastSlash('{app}'+ExtractFilePath(RightStr(ComparedList[I].FilePath,Length(ComparedList[I].FilePath)-1)));
          FilesList[RealFilesListSize,2].Name := 'Components';
          FilesList[RealFilesListSize,2].Value := ComponentName;
          if IssF_FilesAddFlags = true then begin
            FilesList[RealFilesListSize,3].Name := 'Flags';
            FilesList[RealFilesListSize,3].Value := IssF_FilesAddingFlags;
          end;
          //     .
          RealFilesListSize := RealFilesListSize+1;
        end
        else if ComparedList[I].CheckResult = Flist_CheckFile_FileNotExists then begin
          //    .      %).
          //    3   :
          SetLength(DeleteList[RealDeleteListSize],3);
          //,    :
          DeleteList[RealDeleteListSize,0].Name := 'Name';
          DeleteList[RealDeleteListSize,0].Value := DelLastSlash('{app}'+RightStr(ComparedList[I].FilePath,Length(ComparedList[I].FilePath)-1));
          DeleteList[RealDeleteListSize,1].Name := 'Type';
          DeleteList[RealDeleteListSize,1].Value := 'files';
          DeleteList[RealDeleteListSize,2].Name := 'Components';
          DeleteList[RealDeleteListSize,2].Value := ComponentName;
          //     .
          RealDeleteListSize := RealDeleteListSize+1;
        end;
      end
      else if ComparedList[I].FileType = Flist_Filetype_Directory then begin
        //.      ?
        if Length(ComparedList[I].FilePath) > 2 then begin
          //,  ,   .
          //  2  -   ,   .
          if ComparedList[I].CheckResult = Flist_CheckFile_ElementNotExists then begin
            //  ,  .
            //     .
            ElemsNum := 2;  // .
            if IssF_DirsAddFlags = true then begin
              // .
              ElemsNum := ElemsNum+1;
            end;
            //  ...
            SetLength(DirsList[RealDirsListSize],ElemsNum);
            //    ...
            DirsList[RealDirsListSize,0].Name := 'Name';
            DirsList[RealDirsListSize,0].Value := DelLastSlash('{app}'+RightStr(ComparedList[I].FilePath,Length(ComparedList[I].FilePath)-1));
            DirsList[RealDirsListSize,1].Name := 'Components';
            DirsList[RealDirsListSize,1].Value := ComponentName;
            if IssF_DirsAddFlags = true then begin
              DirsList[RealDirsListSize,2].Name := 'Flags';
              DirsList[RealDirsListSize,2].Value := IssF_DirsAddingFlags;
            end;
            //     .
            RealDirsListSize := RealDirsListSize+1;
          end
          else if ComparedList[I].CheckResult = Flist_CheckFile_FileNotExists then begin
            // .      (    ).
            //     ,      
            //            
            //  .
            
            //    3   :
            SetLength(DirDeleteList[RealDirDeleteListSize],3);
            //,    :
            DirDeleteList[RealDirDeleteListSize,0].Name := 'Name';
            DirDeleteList[RealDirDeleteListSize,0].Value := DelLastSlash('{app}'+RightStr(ComparedList[I].FilePath,Length(ComparedList[I].FilePath)-1));
            DirDeleteList[RealDirDeleteListSize,1].Name := 'Type';
            DirDeleteList[RealDirDeleteListSize,1].Value := 'dirifempty';
            DirDeleteList[RealDirDeleteListSize,2].Name := 'Components';
            DirDeleteList[RealDirDeleteListSize,2].Value := ComponentName;
            //     .
            RealDirDeleteListSize := RealDirDeleteListSize+1;
          end;
        end;
      end;
    end;;
  end;
  
  // .   ...
  SetLength(DirsList,RealDirsListSize);
  SetLength(FilesList,RealFilesListSize);
  SetLength(DeleteList,RealDeleteListSize);
  SetLength(DirDeleteList,RealDirDeleteListSize);
  
  //          ...
  Self.AddIniValueList(DirDeleteList,DeleteList);
  
  //      ...
  Self.WriteIniValueList('Dirs',DirsList);
  Self.WriteIniValueList('Files',FilesList);
  Self.WriteIniValueList('InstallDelete',DeleteList);
  
  //.
  Result := true;
  
  // .
  SetLength(ComparedList,0);
  SetLength(DirsList,0,0);
  SetLength(FilesList,0,0);
  SetLength(DeleteList,0,0);
  SetLength(DirDeleteList,0,0);
end;

function TIssFormat.AddNewFilesFromFlists(const OldFlistFileName, NewFlistFileName, ComponentName : AnsiString) : Boolean;
//        FList-   .
var
  OldFlistFile, NewFlistFile : TFlistFormat;
begin
  //...
  Result := false;
  OldFlistFile := TFlistFormat.Create;
  NewFlistFile := TFlistFormat.Create;

  // flist-  -  .
  if (FileExists(OldFlistFileName) = true) and (FileExists(NewFlistFileName) = true) then begin
    // .
    OldFlistFile.Load(OldFlistFileName);
    NewFlistFile.Load(NewFlistFileName);
    //  Flist-.
    OldFlistFile.GlobalRoot := Self.GetSourceDir();
    NewFlistFile.GlobalRoot := Self.GetSourceDir();
    //    .
    Result := Self.AddNewFilesFromFlists(OldFlistFile,NewFlistFile,ComponentName);
  end;

  // ...
  OldFlistFile.Free;
  NewFlistFile.Free;
end;

function TIssFormat.AddNewFilesFromDirs(const OldDirName, NewDirName, ComponentName : AnsiString) : Boolean;
//           .
var
  OldFlistFile, NewFlistFile : TFlistFormat;
begin
  //...
  Result := false;
  OldFlistFile := TFlistFormat.Create;
  NewFlistFile := TFlistFormat.Create;
  
  //...
  OldFlistFile.PathsAreAbsolute := false;
  OldFlistFile.AddSize := true;
  OldFlistFile.AddCRC32 := true;
  OldFlistFile.GlobalRoot := OldDirName;
  NewFlistFile.PathsAreAbsolute := false;
  NewFlistFile.AddSize := true;
  NewFlistFile.AddCRC32 := true;
  NewFlistFile.GlobalRoot := NewDirName;

  //      flist-.
  if (OldFlistFile.AddRealDirectory(OldDirName,true) = true) and (NewFlistFile.AddRealDirectory(NewDirName,true) = true) then begin
    //    -   ...
    OldFlistFile.GlobalRoot := Self.GetSourceDir();
    NewFlistFile.GlobalRoot := Self.GetSourceDir();
    //...      .
    Result := Self.AddNewFilesFromFlists(OldFlistFile,NewFlistFile,ComponentName);
  end;

  // ...
  OldFlistFile.Free;
  NewFlistFile.Free;
end;

function TIssFormat.ReadCodeConstValue(const ConstName : AnsiString) : AnsiString;
//   .
begin
  //.
  Result := '';
  
  // .
  if Self.UpdatePasModifier() = true then begin;
    //  .
    Result := Self.PasModifier.ReadCodeConstValue(ConstName);
  end;
end;

function TIssFormat.WriteCodeConstValue(const ConstName, ConstValue : AnsiString) : boolean;
//   .
begin
  //.
  Result := true;
  
  // .
  if Self.UpdatePasModifier() = true then begin
    //.  .
    Self.PasModifier.WriteCodeConstValue(ConstName, ConstValue);
    //   .
    Result := Self.SaveFromPasModifier();
  end
  else begin
    //,    ,       .
    Result := false;
  end;
end;

function TIssFormat.CodeFunctionExists(const FunctionName : AnsiString) : Boolean;
//     ?
begin
  //.
  Result := false;
  
  // .
  if Self.UpdatePasModifier() = true then begin
    //.      .
    Result := PasModifier.FunctionIsExists(FunctionName);
  end;
end;

function TIssFormat.WriteCodeFunction(const FunctionName, FunctionContent : AnsiString) : Boolean;
//   Code,    - .
begin
  //.
  Result := true;
  
  // .
  if Self.UpdatePasModifier() = true then begin
    //.  .
    if Self.PasModifier.WriteFunction(FunctionName, FunctionContent) = true then begin;
      //   .
      Result := Self.SaveFromPasModifier();
    end
    else begin
      //   .
      Result := false;
      Self.ErrorMessage := 'TPascalCodeModifier - unknown error while trying to write a function '+FunctionName;
    end;
  end
  else begin
    //,    ,       .
    Result := false;
  end;
end;

function TIssFormat.InstallTypeExists(const InstallTypeName : AnsiString) : Boolean;
//   ?
var
  TypesList : AIniValueList;
  I : Integer;
  TempStr : AnsiString;
  
begin
  //.
  Result := false;
  SetLength(TypesList,0,0);
  TempStr := LowerCase(InstallTypeName);
  
  //  ...
  Self.ReadIniValueList('Types',TypesList);
  //  .
  if Length(TypesList) > 0 then begin
    for I := 0 to Length(TypesList)-1 do begin
      if LowerCase(Self.GetIniValueListValue(TypesList,I,'Name')) = TempStr then begin
        //  -   .  .
        Result := true;
        //   .
        Break;
      end;
    end;
  end;
  
  // .
  SetLength(TypesList,0,0);
end;

function TIssFormat.RemoveInstalType(const InstallTypeName : AnsiString) : Boolean;
//  . false        ,   true.
var
  I : Integer;
  TypesList : AIniValueList;
  
begin
  //.
  Result := false;
  SetLength(TypesList,0,0);
  
  //    .
  Self.ReadIniValueList('Types',TypesList);
  //,       ,    .
  I := Self.GetIniValueListIndexByValue(TypesList,'Name',InstallTypeName);
  if (I >= 0) and (Length(TypesList) > 1) then begin
    //   -  .     "(Length(TypesList) > 1)" - 
    //Length(TypesList) = 1     ,    -    
    //      %).
    //    ,  else.
    Self.RemoveIniValueListElement(TypesList,I);
    //          - .
    Self.WriteIniValueList('Types',TypesList);
  end
  else if (I >= 0) and (Length(TypesList) = 1) then begin
    //.  ,     .     .
    Self.RemoveSection('Types');
  end;
  
  // .
  SetLength(TypesList,0,0);
end;

procedure TIssFormat.WriteInstallType(const InstallTypeName, FullInstTypeString : AnsiString);
//  ,   -      . FullInstTypeString -     [Types].
var
  TypesList, TempList : AIniValueList;
  
begin
  //.
  SetLength(TypesList,0,0);
  SetLength(TempList,0,0);
  
  // -     ?
  if Self.InstallTypeExists(InstallTypeName) = true then begin
    //, ,   .
    Self.RemoveInstalType(InstallTypeName);
  end;
  
  //    .
  Self.ReadIniValueList('Types',TypesList);
  //    .
  Self.StrToIniValueList(FullInstTypeString,TempList);
  Self.AddIniValueList(TempList,TypesList);
  //     .
  Self.WriteIniValueList('Types',TypesList);
  
  // .
  SetLength(TypesList,0,0);
  SetLength(TempList,0,0);
end;

function TIssFormat.ComponentExists(const ComponentName : AnsiString) : Boolean;
//   ?
var
  ComponentsList : AIniValueList;
  I : Integer;
  TempStr : AnsiString;

begin
  //.
  Result := false;
  SetLength(ComponentsList,0,0);
  TempStr := LowerCase(ComponentName);

  //  ...
  Self.ReadIniValueList('Components',ComponentsList);
  //  .
  if Length(ComponentsList) > 0 then begin
    for I := 0 to Length(ComponentsList)-1 do begin
      if LowerCase(Self.GetIniValueListValue(ComponentsList,I,'Name')) = TempStr then begin
        //  -   .  .
        Result := true;
        //   .
        Break;
      end;
    end;
  end;

  // .
  SetLength(ComponentsList,0,0);
end;

function TIssFormat.RemoveComponent(const ComponentName : AnsiString) : Boolean;
// . false       ,      true.
var
  I : Integer;
  ComponentsList : AIniValueList;

begin
  //.
  Result := false;
  SetLength(ComponentsList,0,0);

  //   .
  Self.ReadIniValueList('Components',ComponentsList);
  //,      ,    .
  I := Self.GetIniValueListIndexByValue(ComponentsList,'Name',ComponentName);
  if (I >= 0) and (Length(ComponentsList) > 1) then begin
    //   -  .     "(Length(ComponentsList) > 1)" - 
    //Length(ComponentsList) = 1     ,    -    
    //      %).
    //    ,  else.
    Self.RemoveIniValueListElement(ComponentsList,I);
    //          - .
    Self.WriteIniValueList('Components',ComponentsList);
  end
  else if (I >= 0) and (Length(ComponentsList) = 1) then begin
    //.  ,     .     .
    Self.RemoveSection('Components');
  end;

  // .
  SetLength(ComponentsList,0,0);
end;

procedure TIssFormat.WriteComponent(const ComponentName, FullComponentString : AnsiString);
// ,   -      .
//FullComponentString -     [Components].
var
  ComponentsList, TempList : AIniValueList;

begin
  //.
  SetLength(ComponentsList,0,0);
  SetLength(TempList,0,0);

  // -     ?
  if Self.ComponentExists(ComponentName) = true then begin
    //, ,   .
    Self.RemoveComponent(ComponentName);
  end;

  //   .
  Self.ReadIniValueList('Components',ComponentsList);
  //    .
  Self.StrToIniValueList(FullComponentString,TempList);
  Self.AddIniValueList(TempList,ComponentsList);
  //     .
  Self.WriteIniValueList('Components',ComponentsList);

  // .
  SetLength(ComponentsList,0,0);
  SetLength(TempList,0,0);
end;

//----------------------------------------------------------------------------------------//
//          \ -   .        //
//----------------------------------------------------------------------------------------//
function TIssFormat.GetSourceDir() : AnsiString;
// SourceDir.
begin
  Result := Self.ReadString('Setup','SourceDir');
end;

function TIssFormat.GetOutputDir() : AnsiString;
// OutputDir.
begin
  Result := Self.ReadString('Setup','OutputDir');
end;

procedure TIssFormat.SetSourceDir(const SourceDir : AnsiString);
// SourceDir.
begin
  Self.WriteString('Setup','SourceDir', SourceDir);
end;

procedure TIssFormat.SetOutputDir(const OutputDir : AnsiString);
// OutputDir.
begin
  Self.WriteString('Setup','OutputDir', OutputDir);
end;

end.

