///////////////////////////////////////////////////////////
//                     GGBuildTools                      //
//         Gipat Group       //
//             Copyright (C) 2007 Gipat Group            //
//                             //
//    Gipat Group's opened EI-editor-utility license     //
//                       1.0                       //
//                                                       //
//                  www.gipatgroup.org                   //
///////////////////////////////////////////////////////////

//      , ....  :
// 1) Sagrer (sagrer@yandex.ru)

////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
//              VerRevUpdater            //
////////////////////////////////////////////////////////////

unit VerRevUpdServiceClass;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, ExtraFunctionsLcl, GGConsUtilServiceClass, GGBuildToolsShared, ExtraFileUtilsLCL,
  VerRevUpdLogic;
  
const
  GGBT_VerRevUpdaterAppName = ' - Version revision updater';
  
type
  TVerRevUpdServiceClass = class (TGGConsUtilServiceClass)       //      VerRevUpdater
  private
  protected
    LogicClass : TVerRevUpdaterLogic;
    
    // .
    procedure PostExternalMessage(const TextMessage : AnsiString);    //     .
  public
    //
    SconfFilename : AnsiString;

    //-...
    constructor Create; override;
    destructor Destroy; override;

    // ...
    procedure ProcessParameters;  override;     //   .
    procedure DoWork;        override;          //   %).
    procedure ShowDescription; override;        //  .
    procedure ShowOptions;  override;           //    .
  end;
  
var
  VerRevUpdServClass : TVerRevUpdServiceClass;

implementation

/////////////////////////////////////////////
//           TVerRevUpdServClass           //
/////////////////////////////////////////////

//-----------------------------------------//
//        -...      //
//-----------------------------------------//

constructor TVerRevUpdServiceClass.Create;
begin
  //  ...
  SconfFilename := '';

  //   ...
  LogicClass := TVerRevUpdaterLogic.Create;

  //  .
  Self.GGConsAppName := GGBT_VerRevUpdaterAppName;

  //  ...
  inherited;
end;

destructor TVerRevUpdServiceClass.Destroy;
begin
  // 
  LogicClass.Free;

  //  
  inherited;
end;

//------------------------------------------//
//            ...           //
//------------------------------------------//

procedure TVerRevUpdServiceClass.PostExternalMessage(const TextMessage : AnsiString);
//     .
begin
  Writeln(TextMessage);
end;

//------------------------------------------//
//             ...            //
//------------------------------------------//

procedure TVerRevUpdServiceClass.ProcessParameters;
//   .
begin
  //  
  inherited;
  
  if Application.HasOption('filepath') then begin
    //   .
    Self.SconfFilename := Application.GetOptionValue('filepath');
  end;
end;

procedure TVerRevUpdServiceClass.DoWork;
//   %).
begin
  //     -    ProcessFurther := false
  //    .

  //  
  inherited;

  //       -    ....
  if ProcessFurther = true then begin
    //  
    ProcessFurther := false;
    //....
    if Application.HasOption('filepath') = true then begin
      //   .
      //  ...
      Self.ShowVersion();
      Writeln('');
      // -       ...
      if IsGlobalPath(Self.SconfFilename) = false then begin
        //  .     ?
        if IsLocalPath(Self.SconfFilename) = false then begin
          //     - .
          Self.SconfFilename := FixLocalPath(Self.SconfFilename);
        end;
        //    .
        Self.SconfFilename := ResolveLocalPath(Self.SconfFilename,GetCurrentDir);
      end;
      
      //        .    .
      if FileExists(Self.SconfFilename) = true then begin
        //.
        //         .
        LogicClass.PostExternalMessage := @Self.PostExternalMessage;
        if LogicClass.UpdateProjVer(Self.SconfFilename) = false then begin
          //   -  .
          Writeln('Error! '+LogicClass.ErrorMessage);
          Writeln('');
        end
        else begin
          //.
          Writeln('Done.');
          Writeln('');
        end;
      end
      else begin
        //.   .
        Writeln('Error! Project sconf file is not exists! : ');
        Writeln(Self.SconfFilename);
        Writeln('');
      end;
    end
    else begin
      //.       .
      Self.ShowHelp();
      Writeln('Error!!! "--filepath=<value>" option is required!');
      Writeln('');
    end;
  end;

end;

procedure TVerRevUpdServiceClass.ShowDescription;
//  .
begin
  inherited;
  Writeln('Description:');
  Writeln('             Update''s a version of your project. For details - read help pls.');
  Writeln('             To show help message you can use can use "-?" command-line option.');
end;

procedure TVerRevUpdServiceClass.ShowOptions;
//    .
begin
  //  ...
  inherited;
  
  //     %).
  Writeln('          --filepath=<value> : sets path to the project''s configuration file.');
end;

/////////////////////////////////
initialization

// %).  ,   ...     %)
begin
  VerRevUpdServClass := TVerRevUpdServiceClass.Create;
end;

finalization
//  ...
begin
  VerRevUpdServClass.Free;
end;

end.

